/*
 ============================================================================
 Project     : 1.5 Approximation Algorithm for sorting by reversals
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
               Universidade de Brasilia (UnB) - Brazil
 Version     : 1.0
 
 Last Modified On: January 23, 2012.
 ============================================================================
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cycleDecomposition.h"

#include <string>
#include <iostream>
#include <boost/graph/adjacency_list.hpp>
#include <cassert>
#include <boost/graph/max_cardinality_matching.hpp>
using namespace boost;
typedef adjacency_list<vecS, vecS, undirectedS> my_graph; 

/**Gerar a decomposicao de ciclos a partir de G(pi) que esta
 * representado pelo breakpointGraph b**/
void gerarDecomposicaoCiclos(breakpointGraph *b, permutacao *p, decomposicaoSemOrientacao *D, decomposicaoCiclos *C){
	int i, j, k, l, adjPreta, adjCinza1, adjCinza2,temp;
	
	int num2ciclos=0;
	ciclo2 ciclos2[MAX];
	int num2ciclosSemPretas=0;
	int ciclos2SemPretas[MAX];//salvas os indices de 2ciclos sem arestas pretas compartilhadas        
	int num2ciclosSemCinzas=0;
	int ciclos2SemCinzas[MAX];//salvas os indices de 2ciclos sem arestas cinzas compartilhadas        
	ciclo2 auxCiclo2;    
    
	/**Fazer a decomposicao em dois ciclos**/
	//para cada vértice, verificamos se ele faz parte de um 2-ciclo
    for(i=0;i<=p->tamanho+1;i++){
        //para cada aresta preta partindo do no i
        for(j=0;j<b->nadjPreta[i];j++){//para cada adj preta del nodo i
           adjPreta = b->adjPreta[i][j];//adj preta j del nodo i

           for(k=0;k<b->nadjCinza[i];k++){//para cada adj cinza del nodo i
               for(l=0;l<b->nadjCinza[adjPreta];l++){//para cada adj cinza del adjPreta
                   adjCinza1 = b->adjCinza[i][k];///adj cinza k del nodo i
                   adjCinza2 = b->adjCinza[adjPreta][l];///adj cinza l del nodo adjPreta
                   
                   if (abs(adjCinza1-adjCinza2)==1){ /**sao adjacentes**/
					   if (((b->nadjPreta[adjCinza1] == 1)&&(b->adjPreta[adjCinza1][0] == adjCinza2))||
						   ((b->nadjPreta[adjCinza1] == 2)&&(b->adjPreta[adjCinza1][1] == adjCinza2))){					   
						   //checa se esse 2 ciclo ja existe no conjunto de 2 ciclos
						   if(i<adjPreta){
							   auxCiclo2.p1=i;  auxCiclo2.p2=adjPreta;
						   }
						   else{
							   auxCiclo2.p1=adjPreta;  auxCiclo2.p2=i;
						   }

						   if(adjCinza1 < adjCinza2){
							   auxCiclo2.p3=adjCinza1;  auxCiclo2.p4=adjCinza2;
						   }
						   else{
								auxCiclo2.p3=adjCinza2;  auxCiclo2.p4=adjCinza1;
						   }

						   if(i<adjCinza1){
							   auxCiclo2.c1=i;  auxCiclo2.c2=adjCinza1;
						   }
						   else{
							   auxCiclo2.c1=adjCinza1;  auxCiclo2.c2=i;
						   }

						   if(adjPreta<adjCinza2){
							   auxCiclo2.c3=adjPreta;  auxCiclo2.c4=adjCinza2;
						   }
						   else{
							   auxCiclo2.c3=adjCinza2;  auxCiclo2.c4=adjPreta;
						   }

						   if(checaExistencia2Ciclo(ciclos2,num2ciclos,auxCiclo2)==0){
							   ciclos2[num2ciclos++]=auxCiclo2;
						   }
					   }
				   }
               }//End for
           }//End for
        }
    }
    
	/**Criar o matching graph F(pi)**/	
    const int n_vertices = MAX;
	my_graph g(n_vertices);
	
    for(i=0;i<num2ciclos;i++){
		add_edge(ciclos2[i].p1, ciclos2[i].p3, g);
	}
	
	/** Encontrar "the maximun cardinality matching" M,
	    eliminando 2-ciclos que compartilham black edges **/	    
	std::vector<graph_traits<my_graph>::vertex_descriptor> mate(n_vertices);
	checked_edmonds_maximum_cardinality_matching(g, &mate[0]);
	
	graph_traits<my_graph>::vertex_iterator vi, vi_end;
	for(tie(vi,vi_end) = vertices(g); vi != vi_end; ++vi){
		if (mate[*vi] != graph_traits<my_graph>::null_vertex() && *vi < mate[*vi]){
			//std::cout << "{" << *vi << ", " << mate[*vi] << "}" << std::endl;//--------
			for(i=0;i<num2ciclos;i++){
				if ((ciclos2[i].p1 == (int)*vi)&&(ciclos2[i].p3 == (int)mate[*vi])){
					ciclos2SemPretas[num2ciclosSemPretas] = i;
					num2ciclosSemPretas++;
				}
			}
		}
	}
    
	//std::cout << std::endl << "Found a matching of size " << matching_size(g, &mate[0]) << std::endl;
	//std::cout << "The matching is:" << std::endl;
	/**Criar o ladder graph L(M)**/
	my_graph g2(n_vertices);	
	
	int c,d;
	for(i=0;i<num2ciclosSemPretas;i++){
		if (p->pi[ciclos2[ciclos2SemPretas[i]].c1] < p->pi[ciclos2[ciclos2SemPretas[i]].c2])
			c = ciclos2[ciclos2SemPretas[i]].c1;
		else
			c = ciclos2[ciclos2SemPretas[i]].c2;
			
		if (p->pi[ciclos2[ciclos2SemPretas[i]].c3] < p->pi[ciclos2[ciclos2SemPretas[i]].c4])
			d = ciclos2[ciclos2SemPretas[i]].c3;
		else
			d = ciclos2[ciclos2SemPretas[i]].c4;
		
		add_edge(c, d, g2);
		
	}	
		
	/**Eliminar 2-ciclos que compartilham grey edges**/
	std::vector<graph_traits<my_graph>::vertex_descriptor> mate2(n_vertices);
	checked_edmonds_maximum_cardinality_matching(g2, &mate2[0]);
	
	//graph_traits<my_graph>::vertex_iterator vi, vi_end;
	for(tie(vi,vi_end) = vertices(g2); vi != vi_end; ++vi){
		if (mate2[*vi] != graph_traits<my_graph>::null_vertex() && *vi < mate2[*vi]){			
			//std::cout << "*{" << *vi << ", " << mate2[*vi] << "}" << std::endl;//----------
			for(i=0;i<num2ciclos;i++){
				if (p->pi[ciclos2[i].c1] < p->pi[ciclos2[i].c2])
					c = ciclos2[i].c1;
				else
					c = ciclos2[i].c2;
				
				if (p->pi[ciclos2[i].c3] < p->pi[ciclos2[i].c4])
					d = ciclos2[i].c3;
				else
					d = ciclos2[i].c4;
				
				if (c > d){
					temp = c;
					c = d;
					d = temp;
				}				
					
				if (c == (int)*vi && d == (int)mate2[*vi]){					
					ciclos2SemCinzas[num2ciclosSemCinzas] = i;
					num2ciclosSemCinzas++;								
				}
			}
				
		}
	}
	
	/**Eliminar os 2ciclos do breakpointGraph**/
	for(i=0;i<num2ciclosSemCinzas;i++){
		j = ciclos2SemCinzas[i];
		//eliminar black edges p1 e p2
		if (b->nadjPreta[ciclos2[j].p1] == 1){
			b->nadjPreta[ciclos2[j].p1]--;
			b->adjPreta[ciclos2[j].p1][0] = 0;			
		}
		else if (b->nadjPreta[ciclos2[j].p1] == 2){
			b->nadjPreta[ciclos2[j].p1]--;
			if (b->adjPreta[ciclos2[j].p1][0] == ciclos2[j].p2){
				b->adjPreta[ciclos2[j].p1][0] = b->adjPreta[ciclos2[j].p1][1]; 				
			}
			b->adjPreta[ciclos2[j].p1][1] = 0;			
		}
		
		if (b->nadjPreta[ciclos2[j].p2] == 1){
			b->nadjPreta[ciclos2[j].p2]--;
			b->adjPreta[ciclos2[j].p2][0] = 0;			
		}
		else if (b->nadjPreta[ciclos2[j].p2] == 2){
			b->nadjPreta[ciclos2[j].p2]--;
			if (b->adjPreta[ciclos2[j].p2][0] == ciclos2[j].p1){
				b->adjPreta[ciclos2[j].p2][0] = b->adjPreta[ciclos2[j].p2][1]; 				
			}
			b->adjPreta[ciclos2[j].p2][1] = 0;			
		}
		//eliminar black edges p3 e p4
		if (b->nadjPreta[ciclos2[j].p3] == 1){
			b->nadjPreta[ciclos2[j].p3]--;
			b->adjPreta[ciclos2[j].p3][0] = 0;			
		}
		else if (b->nadjPreta[ciclos2[j].p3] == 2){
			b->nadjPreta[ciclos2[j].p3]--;
			if (b->adjPreta[ciclos2[j].p3][0] == ciclos2[j].p4){
				b->adjPreta[ciclos2[j].p3][0] = b->adjPreta[ciclos2[j].p3][1]; 				
			}
			b->adjPreta[ciclos2[j].p3][1] = 0;			
		}
		
		if (b->nadjPreta[ciclos2[j].p4] == 1){
			b->nadjPreta[ciclos2[j].p4]--;
			b->adjPreta[ciclos2[j].p4][0] = 0;			
		}
		else if (b->nadjPreta[ciclos2[j].p4] == 2){
			b->nadjPreta[ciclos2[j].p4]--;
			if (b->adjPreta[ciclos2[j].p4][0] == ciclos2[j].p3){
				b->adjPreta[ciclos2[j].p4][0] = b->adjPreta[ciclos2[j].p4][1]; 				
			}
			b->adjPreta[ciclos2[j].p4][1] = 0;			
		}			
		//eliminar grey edges c1 e c2
		if (b->nadjCinza[ciclos2[j].c1] == 1){
			b->nadjCinza[ciclos2[j].c1]--;
			b->adjCinza[ciclos2[j].c1][0] = 0;			
		}
		else if (b->nadjCinza[ciclos2[j].c1] == 2){
			b->nadjCinza[ciclos2[j].c1]--;
			if (b->adjCinza[ciclos2[j].c1][0] == ciclos2[j].c2){
				b->adjCinza[ciclos2[j].c1][0] = b->adjCinza[ciclos2[j].c1][1]; 				
			}
			b->adjCinza[ciclos2[j].c1][1] = 0;			
		}
		
		if (b->nadjCinza[ciclos2[j].c2] == 1){
			b->nadjCinza[ciclos2[j].c2]--;
			b->adjCinza[ciclos2[j].c2][0] = 0;			
		}
		else if (b->nadjCinza[ciclos2[j].c2] == 2){
			b->nadjCinza[ciclos2[j].c2]--;
			if (b->adjCinza[ciclos2[j].c2][0] == ciclos2[j].c1){
				b->adjCinza[ciclos2[j].c2][0] = b->adjCinza[ciclos2[j].c2][1]; 				
			}
			b->adjCinza[ciclos2[j].c2][1] = 0;			
		}
		//eliminar grey edges c3 e c4
		if (b->nadjCinza[ciclos2[j].c3] == 1){
			b->nadjCinza[ciclos2[j].c3]--;
			b->adjCinza[ciclos2[j].c3][0] = 0;			
		}
		else if (b->nadjCinza[ciclos2[j].c3] == 2){
			b->nadjCinza[ciclos2[j].c3]--;
			if (b->adjCinza[ciclos2[j].c3][0] == ciclos2[j].c4){
				b->adjCinza[ciclos2[j].c3][0] = b->adjCinza[ciclos2[j].c3][1]; 				
			}
			b->adjCinza[ciclos2[j].c3][1] = 0;			
		}
		
		if (b->nadjCinza[ciclos2[j].c4] == 1){
			b->nadjCinza[ciclos2[j].c4]--;
			b->adjCinza[ciclos2[j].c4][0] = 0;			
		}
		else if (b->nadjCinza[ciclos2[j].c4] == 2){
			b->nadjCinza[ciclos2[j].c4]--;
			if (b->adjCinza[ciclos2[j].c4][0] == ciclos2[j].c3){
				b->adjCinza[ciclos2[j].c4][0] = b->adjCinza[ciclos2[j].c4][1]; 				
			}
			b->adjCinza[ciclos2[j].c4][1] = 0;			
		}	
	}
	
	//mostrarBreakpointGraph(b,p);//--		
	/**Fazer a decomposicao com as arestas restantes y adicionar na decomposicao de ciclos**/
	int ptoInicio, encontrado, punto1, punto2;
	
	D->nCiclos = 0;
	encontrado = 1;
	while(encontrado){
		memset(D->ciclo[D->nCiclos].nadjPreta, 0, sizeof(D->ciclo[D->nCiclos].nadjPreta));
		memset(D->ciclo[D->nCiclos].nadjCinza, 0, sizeof(D->ciclo[D->nCiclos].nadjCinza));
		memset(D->ciclo[D->nCiclos].adjPreta, 0, sizeof(D->ciclo[D->nCiclos].adjPreta));
		memset(D->ciclo[D->nCiclos].adjCinza, 0, sizeof(D->ciclo[D->nCiclos].nadjCinza));
		//encontrar punto de inicio
		encontrado = 0;
		for(i=0;i<=p->tamanho+1;i++){
			if (b->nadjCinza[i] > 0){
				ptoInicio = i;
				encontrado = 1;
				break;
			}
		}	
		
		if (encontrado){
			punto1 = ptoInicio;
			punto2 = -1; 
			while (punto2 != ptoInicio){
				
				//Adicionar aresta cinza
				punto2 = b->adjCinza[punto1][b->nadjCinza[punto1]-1];
				
				//printf("Cinza pto1 = %d, pto2 = %d",punto1,punto2);
				
				D->ciclo[D->nCiclos].adjCinza[punto1][D->ciclo[D->nCiclos].nadjCinza[punto1]] = punto2;			
				D->ciclo[D->nCiclos].nadjCinza[punto1]++;			
				
				D->ciclo[D->nCiclos].adjCinza[punto2][D->ciclo[D->nCiclos].nadjCinza[punto2]] = punto1;			
				D->ciclo[D->nCiclos].nadjCinza[punto2]++;						
				//Eliminar aresta cinza
				if (b->nadjCinza[punto1] == 1){
					b->nadjCinza[punto1]--;
					b->adjCinza[punto1][0] = 0;				
				}
				else if (b->nadjCinza[punto1] == 2){
					b->nadjCinza[punto1]--;
					if (b->adjCinza[punto1][0] == punto2){
						b->adjCinza[punto1][0] = b->adjCinza[punto1][1];
					}
					b->adjCinza[punto1][1] = 0;	
				}
				
				if (b->nadjCinza[punto2] == 1){
					b->nadjCinza[punto2]--;
					b->adjCinza[punto2][0] = 0;				
				}
				else if (b->nadjCinza[punto2] == 2){
					b->nadjCinza[punto2]--;
					if (b->adjCinza[punto2][0] == punto1){
						b->adjCinza[punto2][0] = b->adjCinza[punto2][1];
					}
					b->adjCinza[punto2][1] = 0;	
				}			
				//Adicionar aresta preta
				punto1 = punto2;
				punto2 = b->adjPreta[punto1][b->nadjPreta[punto1]-1];
				
				//printf("Cinza pto1 = %d, pto2 = %d",punto1,punto2);
				
				D->ciclo[D->nCiclos].adjPreta[punto1][D->ciclo[D->nCiclos].nadjPreta[punto1]] = punto2;
				D->ciclo[D->nCiclos].nadjPreta[punto1]++;
				
				D->ciclo[D->nCiclos].adjPreta[punto2][D->ciclo[D->nCiclos].nadjPreta[punto2]] = punto1;
				D->ciclo[D->nCiclos].nadjPreta[punto2]++;			
				//Eliminar aresta preta(falta)
				if (b->nadjPreta[punto1] == 1){
					b->nadjPreta[punto1]--;
					b->adjPreta[punto1][0] = 0;				
				}
				else if (b->nadjPreta[punto1] == 2){
					b->nadjPreta[punto1]--;
					if (b->adjPreta[punto1][0] == punto2){
						b->adjPreta[punto1][0] = b->adjPreta[punto1][1];
					}
					b->adjPreta[punto1][1] = 0;	
				}
				
				if (b->nadjPreta[punto2] == 1){
					b->nadjPreta[punto2]--;
					b->adjPreta[punto2][0] = 0;				
				}
				else if (b->nadjPreta[punto2] == 2){
					b->nadjPreta[punto2]--;
					if (b->adjPreta[punto2][0] == punto1){
						b->adjPreta[punto2][0] = b->adjPreta[punto2][1];
					}
					b->adjPreta[punto2][1] = 0;	
				}			
				punto1 = punto2;			
			}
			
			D->nCiclos++;
		}
	}	
	/**Adicionar os 2ciclos na decomposicao de ciclos**/ // Falta Solo esto y testas con test4.txt
	for(i=0;i<num2ciclosSemCinzas;i++){		
		memset(D->ciclo[D->nCiclos].nadjPreta, 0, sizeof(D->ciclo[D->nCiclos].nadjPreta));
		memset(D->ciclo[D->nCiclos].nadjCinza, 0, sizeof(D->ciclo[D->nCiclos].nadjCinza));
		memset(D->ciclo[D->nCiclos].adjPreta, 0, sizeof(D->ciclo[D->nCiclos].adjPreta));
		memset(D->ciclo[D->nCiclos].adjCinza, 0, sizeof(D->ciclo[D->nCiclos].nadjCinza));
		
		j = ciclos2SemCinzas[i];
				
		//adicionar arestas pretas 
		D->ciclo[D->nCiclos].nadjPreta[ciclos2[j].p1]++;
		D->ciclo[D->nCiclos].nadjPreta[ciclos2[j].p2]++;
		D->ciclo[D->nCiclos].adjPreta[ciclos2[j].p1][0] = ciclos2[j].p2;
		D->ciclo[D->nCiclos].adjPreta[ciclos2[j].p2][0] = ciclos2[j].p1;
		
		D->ciclo[D->nCiclos].nadjPreta[ciclos2[j].p3]++;
		D->ciclo[D->nCiclos].nadjPreta[ciclos2[j].p4]++;
		D->ciclo[D->nCiclos].adjPreta[ciclos2[j].p3][0] = ciclos2[j].p4;
		D->ciclo[D->nCiclos].adjPreta[ciclos2[j].p4][0] = ciclos2[j].p3;		
		
		//adicionar arestas cinzas		
		D->ciclo[D->nCiclos].nadjCinza[ciclos2[j].c1]++;
		D->ciclo[D->nCiclos].nadjCinza[ciclos2[j].c2]++;
		D->ciclo[D->nCiclos].adjCinza[ciclos2[j].c1][0] = ciclos2[j].c2;
		D->ciclo[D->nCiclos].adjCinza[ciclos2[j].c2][0] = ciclos2[j].c1;
		
		D->ciclo[D->nCiclos].nadjCinza[ciclos2[j].c3]++;
		D->ciclo[D->nCiclos].nadjCinza[ciclos2[j].c4]++;
		D->ciclo[D->nCiclos].adjCinza[ciclos2[j].c3][0] = ciclos2[j].c4;
		D->ciclo[D->nCiclos].adjCinza[ciclos2[j].c4][0] = ciclos2[j].c3;		
		 
		D->nCiclos++; 
	}
	
    //gera a orientacao do ciclo
    orientacaoCiclos(D, p, C);
    
}

int checaExistencia2Ciclo(ciclo2 c[],int n, ciclo2 aux){
    int i;

    for(i=0;i<n;i++){
        if(aux.p1 == c[i].p1 && aux.p2 == c[i].p2 &&
            aux.p3==c[i].p3 && aux.p4==c[i].p4){
            return(1);
        }
        else if(aux.p3 == c[i].p1 && aux.p4 == c[i].p2 &&
                aux.p1 == c[i].p3 && aux.p2 == c[i].p4){
            return(1);
        }
    }
    return(0);
}

void orientacaoCiclos(decomposicaoSemOrientacao *D, permutacao *p, decomposicaoCiclos *C){
    
    int i,j,ptoPartida, preta1,preta2,cinza1,cinza2,tempCinza0,tempCinza1,difPretas;
    int aCinza,pPreto,pretaFinal, encontrado;
    breakpointGraph ciclo;   
    
    C->nCiclos = 0;
    for(i=0; i<D->nCiclos; i++){
        
        ciclo = D->ciclo[i];
        encontrado = 1;
        
        while (encontrado){           
            //encontrar pto de partida
            ptoPartida = -1;
            encontrado = 0;
            for(j=0; j<=p->tamanho+1; j++){
                if(ciclo.nadjPreta[j] > 0){
                    ptoPartida = j; 
                    encontrado = 1;
                    break;
                }
            }
            
            if (encontrado){                                
                memset(C->ciclo[C->nCiclos].arestaCinza, -1, sizeof(C->ciclo[C->nCiclos].arestaCinza));//@  #
                //encontrar orientacao do ciclo atual
                //encontrar preta e cinza inicial
                preta1 = ptoPartida;
                preta2 = ciclo.adjPreta[ptoPartida][0];
                pretaFinal = ciclo.adjPreta[ptoPartida][0];//@ usado para completar matriz arestaCinza#
                
                cinza1 = preta2;
                if (ciclo.nadjCinza[cinza1]==1){
                    cinza2 = ciclo.adjCinza[cinza1][0];            
                }
                else{//==2
                    difPretas = preta1 - preta2;//sentido del ciclo
                    
                    tempCinza0 = ciclo.adjCinza[cinza1][0];
                    tempCinza1 = ciclo.adjCinza[cinza1][1];        
                    cinza2 = tempCinza0; 
                    
                    /*if ((ciclo.nadjPreta[tempCinza0] == 1)&&
                     (tempCinza0 - ciclo.adjPreta[tempCinza0][0] != difPretas)){
                     cinza2 = tempCinza0; 
                     }
                     else */if ((ciclo.nadjPreta[tempCinza1] == 1)&&
                                (tempCinza1 - ciclo.adjPreta[tempCinza1][0] != difPretas)){
                         cinza2 = tempCinza1; 
                     }
                     else if (ciclo.nadjPreta[tempCinza0] == 2){
                         cinza2 = tempCinza0; 
                     }
                     else if (ciclo.nadjPreta[tempCinza1] == 2){
                         cinza2 = tempCinza1;  
                     }
                }
                                
                //@
                if (p->pi[cinza1] < p->pi[cinza2]){
                    aCinza = p->pi[cinza1];
                    pPreto = p->pi[preta1];
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][0] = aCinza + 1;//Adicionar aresta cinza aCinza-aCinza+1
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][1] = pPreto;//Adicionar preta relacionada con aCinza
                }
                else{
                    aCinza = p->pi[cinza2];
                    pPreto = p->pi[preta1];
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][0] = aCinza + 1;//Adicionar aresta cinza aCinza-aCinza+1
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][2] = pPreto;//Adicionar preta relacionada con aCinza+1
                }
                //#
                
                //eliminar de ciclo temporal preta e cinza
                eliminarAdjPreta(&ciclo,preta1,preta2);
                eliminarAdjCinza(&ciclo,cinza1,cinza2);
                
                //****encontrar as demais pretas e cinzas
                while(cinza2 != ptoPartida){
                    difPretas = preta1 - preta2;//sentido del ciclo            
                    //encontrar preta 
                    preta1 = cinza2;
                    if (ciclo.nadjPreta[preta1] == 1){
                        preta2 = ciclo.adjPreta[preta1][0];
                    }
                    else{ // == 2
                        if (preta1-ciclo.adjPreta[preta1][0] != difPretas){
                            preta2 = ciclo.adjPreta[preta1][0];                           
                        }
                        else{
                            preta2 = ciclo.adjPreta[preta1][1]; 
                        }
                    }
                    
                    //encontrar cinza
                    cinza1 = preta2;
                    if (ciclo.nadjCinza[cinza1]==1){
                        cinza2 = ciclo.adjCinza[cinza1][0];            
                    }
                    else{//==2
                        difPretas = preta1 - preta2;//sentido del ciclo
                        
                        tempCinza0 = ciclo.adjCinza[cinza1][0];
                        tempCinza1 = ciclo.adjCinza[cinza1][1];        
                        cinza2 = tempCinza0; 
                        
                        /*if ((ciclo.nadjPreta[tempCinza0] == 1)&&
                         (tempCinza0 - ciclo.adjPreta[tempCinza0][0] != difPretas)){
                         cinza2 = tempCinza0; 
                         }
                         else*/ if ((ciclo.nadjPreta[tempCinza1] == 1)&&
                                    (tempCinza1 - ciclo.adjPreta[tempCinza1][0] != difPretas)){
                             cinza2 = tempCinza1; 
                         }
                         else if (ciclo.nadjPreta[tempCinza0] == 2){
                             cinza2 = tempCinza0; 
                         }
                         else if (ciclo.nadjPreta[tempCinza1] == 2){
                             cinza2 = tempCinza1;  
                         }
                    }
                    
                    //@
                    if (p->piInverso[aCinza] == preta1){//falta aresta preta para aCinza
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][1] = p->pi[preta2];//Adicionar preta relacionada con aCinza             
                    }
                    else{//falta aresta preta para aCinza + 1
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][2] = p->pi[preta2];//Adicionar preta relacionada con aCinza+1
                    }
                    //#
                    
                    //@
                    if (p->pi[cinza1] < p->pi[cinza2]){
                        aCinza = p->pi[cinza1];
                        pPreto = p->pi[preta1];
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][0] = aCinza + 1;//Adicionar aresta cinza aCinza-aCinza+1
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][1] = pPreto;//Adicionar preta relacionada con aCinza
                    }
                    else{
                        aCinza = p->pi[cinza2];
                        pPreto = p->pi[preta1];
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][0] = aCinza + 1;//Adicionar aresta cinza aCinza-aCinza+1
                        C->ciclo[C->nCiclos].arestaCinza[aCinza][2] = pPreto;//Adicionar preta relacionada con aCinza+1
                    }
                    //#
                    
                    //eliminar de ciclo temporal preta e cinza
                    eliminarAdjPreta(&ciclo,preta1,preta2);
                    eliminarAdjCinza(&ciclo,cinza1,cinza2);          
                    
                }//FIM WHILE
                
                //@
                if (p->piInverso[aCinza] == cinza2){//falta aresta preta para aCinza
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][1] = p->pi[pretaFinal];//Adicionar preta relacionada con aCinza             
                }
                else{//falta aresta preta para aCinza + 1
                    C->ciclo[C->nCiclos].arestaCinza[aCinza][2] = p->pi[pretaFinal];//Adicionar preta relacionada con aCinza+1
                }
                //#
                
                C->nCiclos++;
            }//end if
        }//end while
		
	}//end for
}

void eliminarAdjPreta(breakpointGraph *ciclo,int preta1, int preta2){
    if (ciclo->nadjPreta[preta1] == 1){
        ciclo->nadjPreta[preta1]--;
        ciclo->adjPreta[preta1][0] = 0;				
    }
    else if (ciclo->nadjPreta[preta1] == 2){
        ciclo->nadjPreta[preta1]--;
        if (ciclo->adjPreta[preta1][0] == preta2){
            ciclo->adjPreta[preta1][0] = ciclo->adjPreta[preta1][1];
        }
        ciclo->adjPreta[preta1][1] = 0;	
    }
    
    if (ciclo->nadjPreta[preta2] == 1){
        ciclo->nadjPreta[preta2]--;
        ciclo->adjPreta[preta2][0] = 0;				
    }
    else if (ciclo->nadjPreta[preta2] == 2){
        ciclo->nadjPreta[preta2]--;
        if (ciclo->adjPreta[preta2][0] == preta1){
            ciclo->adjPreta[preta2][0] = ciclo->adjPreta[preta2][1];
        }
        ciclo->adjPreta[preta2][1] = 0;	
    }
    
}

void eliminarAdjCinza(breakpointGraph *ciclo,int cinza1, int cinza2){
    if (ciclo->nadjCinza[cinza1] == 1){
        ciclo->nadjCinza[cinza1]--;
        ciclo->adjCinza[cinza1][0] = 0;				
    }
    else if (ciclo->nadjCinza[cinza1] == 2){
        ciclo->nadjCinza[cinza1]--;
        if (ciclo->adjCinza[cinza1][0] == cinza2){
            ciclo->adjCinza[cinza1][0] = ciclo->adjCinza[cinza1][1];
        }
        ciclo->adjCinza[cinza1][1] = 0;	
    }
    
    if (ciclo->nadjCinza[cinza2] == 1){
        ciclo->nadjCinza[cinza2]--;
        ciclo->adjCinza[cinza2][0] = 0;				
    }
    else if (ciclo->nadjCinza[cinza2] == 2){
        ciclo->nadjCinza[cinza2]--;
        if (ciclo->adjCinza[cinza2][0] == cinza1){
            ciclo->adjCinza[cinza2][0] = ciclo->adjCinza[cinza2][1];
        }
        ciclo->adjCinza[cinza2][1] = 0;	
    }
    
}





