/*
 ============================================================================
 Project     : 1.5 Approximation Algorithm for sorting by reversals
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
               Universidade de Brasilia (UnB) - Brazil
 Version     : 1.0
 
 Last Modified On: January 23, 2012.
 ============================================================================
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "eliminacao.h"
#include "reversalGraph.h"
#include "cycleDecomposition.h"
#include "breakpointGraph.h"
#include "main.h"

/**
* Retorna o conjunto de reversoes
**/
void eliminacao(permutacao *p, reversals* revs){
	int i,j,k;
    int r=0;
    int verticesComponente[MAX];
    char marcado2[MAX];
    int resposta;    
    
    reversalGraph copia;    
    revs->reversoes = (int*)malloc(sizeof(int)*MAX);//(int*) para g++
    revs->intervalos = (intervalo*)malloc(sizeof(intervalo)*MAX);//(intervalo*) para g++
    
    int verticesIsolados = -1;
    int tamanho = 0;
    
    /**enquanto todos os vertices nao forem isolados**/
    while(verticesIsolados < tamanho){
                    
        //gerar
        //DECOMPOSICAO de CICLOS e GRAFO de REVERSALS
        breakpointGraph bpg;
        reversalGraph g;	
        decomposicaoSemOrientacao D;
        decomposicaoCiclos C;
            
        p->tamanho -= 2;  
        montaGrafoBreakpoint(p, &bpg);        
        gerarDecomposicaoCiclos(&bpg, p,&D,&C);
        p->tamanho += 2;   
        gerarReversalGraph(&C, p, &g);
        
        //vExibeRC(&g);
        
        //ELIMINACAO
        preProcessa(&g);
    
        for(i=0;i<g.tamanho;i++){
            if(g.orientacaoComponente[i]== DESORIENTADA && i != 0 && i != p->tamanho-2){            
                //printf("**0-reversal\n");	
                aplicaReversao(&g,i,&C);
                revs->reversoes[r]=i; //Esto tambien deberia estar aqui            
                atualizarCicloDecomposicao(p,&C,r,i,revs,g.cor[i]);//Esto lo deberia aplicar tambien
                r++;
                
                preProcessa(&g);                       
            }
        }
            
        //vExibeRC(g);// Exibe o Reversal Graph
        memset(verticesComponente,0,sizeof(verticesComponente));
        
        //pegar un componente con adjancentes
        for(i=0;i<g.tamanho;i++){
            if(g.nadj[i]!=0){
                break;
            }
        }
        
           
        //recupera todos los vertices que estan en la misma componente que vertices i
        for(j=0;j<g.tamanho;j++){
            if(g.componente[j] == g.componente[i]){
                verticesComponente[j] = 1;
            }
        }
        
        //para cada vrtice da mesma componente do vrtice i
        for(j=0;j<g.tamanho;j++){
            if(g.componente[j]==g.componente[i] && g.cor[j]==VERMELHO){
                //printf("Escolhendo o vertice %d para aplicar a reversao\n",j);//--
                copia = g;
                aplicaReversao(&g,j,&C);
                resposta = ORIENTADA;
                for(k=0;k<g.tamanho;k++){
                    if(verticesComponente[k]==1){
                        memset(marcado2,0,sizeof(marcado2));
                        if(verificaOrientacao(&g,k,marcado2)==DESORIENTADA){
                            resposta = DESORIENTADA;
                            break;
                        }
                    }
                }
                if(resposta==ORIENTADA){							
                    //printf("Reversao relativa ao vertice %d gerou so componentes orientadas\n",j);//--
                    revs->reversoes[r]=j;
                    atualizarCicloDecomposicao(p,&C,r,j,revs,VERMELHO);//						
                    r++;
                    break;
                }
                else{
                    //printf("Reversao relativa ao vertice %d gerou componentes desorientadas, voltando para o estado anterior\n",j);//--
                    g = copia;
                }
            }
        }//end for
            
        verticesIsolados = g.verticesIsolados; 
        tamanho = g.tamanho; 
                   
        
    }//fin while
    revs->tamanho = r;
}

void atualizarCicloDecomposicao(permutacao *p, decomposicaoCiclos *C,int posReversao, int vertice, 
                                reversals* revs,int cor){
	
	int j,atual,grey1, grey2, temp, black1, black2, direita,doisCiclo, a, b;
	//int nroCiclos = 2;//pasar como parametro	
	
	/**atualizar o ciclo de decomposicao**/
	if (cor == VERMELHO){
		/**encontrar a que ciclo de decomposicao pertence o vertice**/
		grey1 = p->piInverso[vertice];
		grey2 = p->piInverso[vertice+1];
		
		atual = -1;//so inicializacao 
        
        for(j=0; j<C->nCiclos; j++){
            if (C->ciclo[j].arestaCinza[vertice][0] == vertice+1){
                atual = j;
                break;
            }
        }
        
		/**verificar se os black edges estao a direita o esquerda do grey edge**/		
		if (grey2 < grey1){
			temp = grey1;
			grey1 = grey2;
			grey2 = temp;
		}		
		//if (C->ciclo[atual].nadjPreta[grey1] > 0)// um ciclo sempre existira pelo menos aresta preta
		black1 = escolherArestaPreta(grey1, grey2, atual, C,p);//C->ciclo[atual].adjPreta[grey1][0];
		//if (C->ciclo[atual].nadjPreta[grey2] > 0)// um ciclo sempre existira pelo menos aresta preta
		black2 = escolherArestaPreta(grey2, grey1, atual, C,p);//C->ciclo[atual].adjPreta[grey2][0];		
		
		direita = 0;// esquerda
        
		if ((black1 > grey1)&&(black2 > grey2))
			direita = 1;// direita
		/**verificar se ciclo atual  um 2-ciclo**/
		doisCiclo=0;		
        if (p->pi[black1] < p->pi[black2]){
            if (p->pi[black1]+1 == p->pi[black2])
                doisCiclo =1;            
        }
        else{//p->pi[black1] > p->pi[black2]
            if (p->pi[black2]+1 == p->pi[black1])
                doisCiclo =1;
        }
        
		/**gerar intervalo de reversao (entre dois black edges)**/
		if (direita){
			a = black1;
			b = grey2;
		}
		else{//esquerda
			a = grey1;
			b = black2;
		}	
        
		/**modificar ciclo de decomposicao atual**/
		//printf("MODIFICAR CICLO DE DECOMPOSICAO ATUAL\n");	
        
        //eliminar aresta cinza (grey1-grey2)
        if (p->pi[grey1] < p->pi[grey2]){
            C->ciclo[atual].arestaCinza[p->pi[grey1]][0] = -1;
            C->ciclo[atual].arestaCinza[p->pi[grey1]][1] = -1;
            C->ciclo[atual].arestaCinza[p->pi[grey1]][2] = -1;
        }            
        else{//p->pi[grey1] > p->pi[grey2]
            C->ciclo[atual].arestaCinza[p->pi[grey2]][0] = -1;
            C->ciclo[atual].arestaCinza[p->pi[grey2]][1] = -1;
            C->ciclo[atual].arestaCinza[p->pi[grey2]][2] = -1;
        }
        //actualizar o par cinza-preta black2-grey2 com black2-black1
        for(j=0; j < p->tamanho-1; j++){
            if ((j == p->pi[black2])&&
                (C->ciclo[atual].arestaCinza[j][1] == p->pi[grey2])){
                
                C->ciclo[atual].arestaCinza[j][1] = p->pi[black1];
                break;
            }
            if ((C->ciclo[atual].arestaCinza[j][0] == p->pi[black2])&&
                (C->ciclo[atual].arestaCinza[j][2] == p->pi[grey2])){
                
                C->ciclo[atual].arestaCinza[j][2] = p->pi[black1];
                break;
            }
        }            
        //actualizar o par cinza-preta black1-grey1 com black1-black2
        for(j=0; j < p->tamanho-1; j++){
            if ((j == p->pi[black1])&&
                (C->ciclo[atual].arestaCinza[j][1] == p->pi[grey1])){
                
                C->ciclo[atual].arestaCinza[j][1] = p->pi[black2];
                break;
            }
            if ((C->ciclo[atual].arestaCinza[j][0] == p->pi[black1])&&
                (C->ciclo[atual].arestaCinza[j][2] == p->pi[grey1])){
                
                C->ciclo[atual].arestaCinza[j][2] = p->pi[black2];
                break;
            }
        }
        //Si es 2-ciclo eliminar aresta restante (black1-black2)
        if (doisCiclo){ 
            if (p->pi[black1] < p->pi[black2]){
                C->ciclo[atual].arestaCinza[p->pi[black1]][0] = -1;
                C->ciclo[atual].arestaCinza[p->pi[black1]][1] = -1;
                C->ciclo[atual].arestaCinza[p->pi[black1]][2] = -1;
            }            
            else{//p->pi[black1] > p->pi[black2]
                C->ciclo[atual].arestaCinza[p->pi[black2]][0] = -1;
                C->ciclo[atual].arestaCinza[p->pi[black2]][1] = -1;
                C->ciclo[atual].arestaCinza[p->pi[black2]][2] = -1;
            }
        }
        
    }
    else{//cor == BLUE
        grey1 = p->piInverso[vertice];
		grey2 = p->piInverso[vertice+1];
        //actualizar arestas
        if (grey1 < grey2){
            black1 = grey1-1;
            black2 = grey2+1;           
            //intervalo de reversao
            a = grey1;
            b = grey2;            
        }
        else{
            black1 = grey1+1;
            black2 = grey2-1;
            
            //intervalo de reversao
            a = grey2;
            b = grey1;
        }
        
        //Actualizar grey1-black1 a grey1-black2
        for(j=0; j<C->nCiclos; j++){
            if (C->ciclo[j].arestaCinza[p->pi[grey1]][1] == p->pi[black1]){
                C->ciclo[j].arestaCinza[p->pi[grey1]][1] = p->pi[black2];
            }
            if (C->ciclo[j].arestaCinza[p->pi[grey1]-1][2] == p->pi[black1]){
                C->ciclo[j].arestaCinza[p->pi[grey1]-1][2] = p->pi[black2];
            }
        } 
        //Actualizar grey2-black2 a grey2-black1
        for(j=0; j<C->nCiclos; j++){
            if (C->ciclo[j].arestaCinza[p->pi[grey2]][1] == p->pi[black2]){
                C->ciclo[j].arestaCinza[p->pi[grey2]][1] = p->pi[black1];
            }
            if (C->ciclo[j].arestaCinza[p->pi[grey2]-1][2] == p->pi[black2]){
                C->ciclo[j].arestaCinza[p->pi[grey2]-1][2] = p->pi[black1];
            }
        } 
        
        //Actualizar black1-grey1 a black1-grey2
        for(j=0; j<C->nCiclos; j++){
            if (C->ciclo[j].arestaCinza[p->pi[black1]][1] == p->pi[grey1]){
                C->ciclo[j].arestaCinza[p->pi[black1]][1] = p->pi[grey2];
            }
            if (C->ciclo[j].arestaCinza[p->pi[black1]-1][2] == p->pi[grey1]){
                C->ciclo[j].arestaCinza[p->pi[black1]-1][2] = p->pi[grey2];
            }
        } 
        //Actualizar black2-grey2 a black2-grey1
        for(j=0; j<C->nCiclos; j++){
            if (C->ciclo[j].arestaCinza[p->pi[black2]][1] == p->pi[grey2]){
                C->ciclo[j].arestaCinza[p->pi[black2]][1] = p->pi[grey1];
            }    
            if (C->ciclo[j].arestaCinza[p->pi[black2]-1][2] == p->pi[grey2]){
                C->ciclo[j].arestaCinza[p->pi[black2]-1][2] = p->pi[grey1];
            }
        } 
        
        
    } 
	
    revs->intervalos[posReversao].esq = a;
    revs->intervalos[posReversao].dir = b;		
    
	/**aplicar reversao sobre a permutacao pi e piInverso**/
	reverte(p,a,b);	
    
}

/** Aplica a reversao dada pelo vertice v e faz as modificacoes
no grafo de reversoes, retorna o numero de componentes orientadas.
**/
void aplicaReversao(reversalGraph* g, int v ,decomposicaoCiclos *C){
    int i,j;
    int componente;
    int marcado[MAX];
    /**O(n)**/
    /** se temos (v,i) e (v,j), (i,j)  flipada**/
    for(i=0;i<g->tamanho;i++){
        for(j=i+1;j<g->tamanho;j++){

            if(g->adj[v][i] == 1 && g->adj[v][j]==1){
                flipaAresta(g,i,j,C);
            }
        }
    }

    /**flipa a cor de todos os adjacentes a reversao representada por v**/
    for(i=0;i<g->tamanho;i++){
        if(g->adj[v][i]==1){
            flipaCor(g,i);
        }
    }

    /**se vermelho, torna um vertice isolado**/
    if(g->cor[v] == VERMELHO){
        flipaCor(g,v);
        /**isola o vrtice**/
        for(i=0;i<g->tamanho;i++){
            g->adj[v][i]=0;
        }
        g->nadj[v]=0;
        for(i=0;i<g->tamanho;i++){
			if(g->adj[i][v]==1){
				g->adj[i][v]=0;
				g->nadj[i]--;
			}
		}
        memset(marcado,0,sizeof(marcado));
        componente = g->componente[v];
        /**para cada vrtice da mesma componente de v, atualizamos a informacao**/
        for(i=0;i<g->tamanho;i++){
            if(g->componente[i] == componente){
                marcado[i] = 1;
                g->componente[i] = i;
            }
        }
        for(i=0;i<g->tamanho;i++){
            if(marcado[i]){
                dfsReversal(g,i,g->componente[i]);
            }
        }
    }
    
    g->verticesIsolados=0;
    for(i=0;i<g->tamanho;i++){
		if(g->nadj[i]==0){
			g->verticesIsolados++;
		}
	}	
}

void flipaAresta(reversalGraph* g,int u,int v,decomposicaoCiclos *C){
    int j, atual, casoEspecial;
    
    //encontrar a que ciclo de decomposicao pertence o vertice
    atual = -1;//so inicializacao    
    for(j=0; j<C->nCiclos; j++){
        if (C->ciclo[j].arestaCinza[u][0] == u+1){
            atual = j;
            break;
        }
    }
    
    //verificar caso especial do lema 4.1
    casoEspecial = 0;
    if (C->ciclo[atual].arestaCinza[v][0] == v+1){//si otro vertice esta en el mismo ciclo
        if (u < v){
            if (u+1 == v && //verificar que son consecutivos e
                C->ciclo[atual].arestaCinza[u][1] == v+1 && // que existe aresta preta entre u e v+1, e
                C->ciclo[atual].arestaCinza[v][2] == u){ // que existe aresta preta entre v+1  e u               
                casoEspecial = 1;
            }
        }
        else{ //u > v
            if (v+1 == u && //verificar que son consecutivos e
                C->ciclo[atual].arestaCinza[v][1] == u+1 && // que existe aresta preta entre v e u+1,e
                C->ciclo[atual].arestaCinza[u][2] == v){ // que existe aresta preta entre u+1 e v
                casoEspecial = 1;
            }
        }
    }
     
    
    //flipar as arestas si no es caso especial
    if (!casoEspecial){    
        if(g->adj[u][v]==1){
            g->adj[u][v]=0;
            g->adj[v][u]=0;
            g->nadj[u]--;
            g->nadj[v]--;
        }
        else{
            g->adj[u][v]=1;
            g->adj[v][u]=1;
            g->nadj[u]++;
            g->nadj[v]++;
        }
    }
}

void flipaCor(reversalGraph* g,int v){
    g->cor[v] = g->cor[v]^1; //invertir de 0 a 1 o de 1 a 0. 
}

/**aplica a busca em profundidade e marca os vertices
que estao na mesma componente conexa do lider**/
void dfsReversal(reversalGraph* g,int v,int lider){
    int i;
    g->componente[v] = lider;
    for(i=0;i<g->tamanho;i++){
        if(g->adj[v][i]==1 && g->componente[i]!=lider){
            dfsReversal(g,i,lider);
        }
    }
}

int verificaOrientacao(reversalGraph *g,int v,char marcado[]){
	int i;

	marcado[v]=1;
    if(g->nadj[v]==0){
        return(ORIENTADA);
    }
    else{
        if(g->cor[v]==VERMELHO){
            return(ORIENTADA);
        }
        for(i=0;i<g->tamanho;i++){
            if(g->adj[v][i] && marcado[i]==0){
                if(verificaOrientacao(g,i,marcado)==ORIENTADA){
                    return(ORIENTADA);
                }
            }
        }
        return(DESORIENTADA);
    }
}

/**preprocessa as componentes conexas e no conexas**/
void preProcessa(reversalGraph* g){
    int i,j;
    int orientacao;
    char marcado[MAX];
    char marcado2[MAX];
    for(i=0;i<g->tamanho;i++){
        g->componente[i]=i;
    }
    /**indica as componentes conexas**/
    for(i=0;i<g->tamanho;i++){
        dfsReversal(g,i,g->componente[i]);
    }

	memset(marcado,0,sizeof(marcado));
    for(i=0;i<g->tamanho;i++){
		if(!marcado[i]){
			marcado[i]=1;
			memset(marcado2,0,sizeof(marcado2));
			orientacao = verificaOrientacao(g,i,marcado2);
			g->orientacaoComponente[i] = orientacao;
			for(j=0;j<g->tamanho;j++){
				if(g->componente[j] == g->componente[i]){
					g->orientacaoComponente[j] = g->orientacaoComponente[i];
					marcado[j]=1;
				}
			}
		}
    }

    /**conta o numero de vertices azuis isolados**/
    g->verticesIsolados=0;
    for(i=0;i<g->tamanho;i++){
        if(g->nadj[i]==0){
            g->verticesIsolados++;
        }
    }
}

void reverte(permutacao* p,int a,int b){
	int aux,i;
	for(i=0;i<(b-a+1)/2;i++){ 
        aux = p->pi[a+i]; 
        p->pi[a+i] = p->pi[b-i]; 
        p->pi[b-i] = aux; 

		p->piInverso[p->pi[a+i]] = a+i;//b-i; //usamos los mismos valores, 
        p->piInverso[p->pi[b-i]] = b-i;//a+i; //por que ya se modifico pi        		
    }
}






