/*
 ============================================================================
 Project     : 1.5 Approximation Algorithm for sorting by reversals
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
               Universidade de Brasilia (UnB) - Brazil
 Version     : 1.0
 
 Last Modified On: January 23, 2012.
 ============================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>//para medir tiempo
#include "main.h"
#include "breakpointGraph.h"
#include "cycleDecomposition.h"
#include "reversalGraph.h"
#include "eliminacao.h"

int main(int argc, char** argv)
{	
    struct timeval t_ini, t_fin;
    double secs;
    
    gettimeofday(&t_ini, NULL);//Tomar TIEMPO de inicio
    
	permutacao p;
	reversals rev;	
	int i;

    /* Carga da permutacao */
    //printf("Entre com o tamanho da permutacao: ");
    printf("** 1.5 Approximation Algorithm **\n");
    printf("Enter permutation length:");
    scanf("%d",&p.tamanho);

    //printf("Entre com a permutacao: ");
    p.pi[0]=0;
    p.pi[p.tamanho+1]=p.tamanho+1;
    for(i=1;i<=p.tamanho;i++){
        //printf("pi[%d]? ",i);
        printf("pi[%d]? ",i);
        scanf("%d",&p.pi[i]);
    }    

    /* Exibicao da permutacao carregada */
    //printf("Permutao completa montada:\n");
    printf("\n Initial permutation:"); 
    vExibePermutacao(p.pi, p.tamanho);

    /* Geracao e exibicao da permutacao inversa */
    vGeraPIInverso(&p);
    
    p.tamanho += 2;
    eliminacao(&p,&rev);
    p.tamanho -= 2;
    
    gettimeofday(&t_fin, NULL);//Tomar TIEMPO final    
    secs = timeval_diff(&t_fin, &t_ini);//Calcular tiempo total usado
    
    //mostraReversos(&rev,&pcopia);        
    mostrarResultado(&p,&rev,secs);
    
    return 0;
}

void mostrarResultado(permutacao *p, reversals *rev, double secs){
	int i;    
	printf("Final permutation:"); vExibePermutacao(p->pi, p->tamanho);
	//printf("Number of breakpoints:%d\n",nroBreakpoints(p));
	printf("Number of reversals: %d\n",rev->tamanho);
	printf("Reversals(positions):"); 
	for(i=0; i<rev->tamanho-1; i++)
		printf("(%d,%d),",rev->intervalos[i].esq,rev->intervalos[i].dir);
	printf("(%d,%d)\n",rev->intervalos[i].esq,rev->intervalos[i].dir);
	printf("Total time: %.16g milliseconds\n", secs * 1000.0);
    //printf("VersionUltima 23Enero\n");
    printf("\n");
}

double timeval_diff(struct timeval *a, struct timeval *b)
{
    return
    (double)(a->tv_sec + (double)a->tv_usec/1000000) -
    (double)(b->tv_sec + (double)b->tv_usec/1000000);
}

int nroBreakpoints(permutacao *p){
	int i,nroBp=0;
	for(i=0; i<=p->tamanho; i++){
		if (abs(p->pi[i] - p->pi[i+1]) > 1)
			nroBp++;		
	}
	return nroBp;
}

void vExibePermutacao(const int *p, int tamanho){
int i;
	for (i=0 ; i<= tamanho+1; i++)
		printf("%d ", p[i]);
	printf("\n");
}

void vGeraPIInverso(permutacao *p){
int i;
	for (i=0 ; i<= p->tamanho+1; i++)
		p->piInverso[p->pi[i]]=i;
}





