/*
 ============================================================================
 Project     : 1.5 Approximation Algorithm for sorting by reversals
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
               Universidade de Brasilia (UnB) - Brazil
 Version     : 1.0
 
 Last Modified On: January 23, 2012.
 ============================================================================
 
 */

#include <stdlib.h>
#include <string.h>
#include "reversalGraph.h"

void gerarReversalGraph(decomposicaoCiclos *C, permutacao *p, reversalGraph *revGraph){
    int i, j, k, f, g;

	/* JP: Inicializacoes */
	revGraph->verticesIsolados=0;
	revGraph->tamanho=0;
	memset(revGraph->adj,0,sizeof(revGraph->adj));
	memset(revGraph->nadj,0,sizeof(revGraph->nadj));

	/**Gerar vertices isolados**/
	for(i=0; i<p->tamanho-1; i++){
		if (abs(p->pi[i] - p->pi[i+1]) == 1){//no  black edge
			revGraph->cor[p->pi[i]] = AZUL;//pintar de cor azul
			revGraph->verticesIsolados++;
		}
	}	
			
	/**Gerar vertices vermelhos e azuis**/	
    for(i=0; i<C->nCiclos; i++){
        for(j=0; j<p->tamanho-1; j++){
            if (C->ciclo[i].arestaCinza[j][0] == j+1)
                calcularCorVertice(j, i, C, p, revGraph);
        }    
    }

	/* JP: corrige a contagem, pois esta dobrada e ainda nao considera os vertices isolados */
	//revGraph->tamanho = revGraph->tamanho/2;
	revGraph->tamanho += revGraph->verticesIsolados;
    	
	/**Gerar arestas**/	
	//comparar dentro dos mesmos ciclos	
    for(i=0; i<C->nCiclos; i++){				
        
		for(j=0; j<p->tamanho-2; j++){
			if (C->ciclo[i].arestaCinza[j][0] == j+1){//se vertice j forma uma aresta                
				
                for(k=j+1; k<p->tamanho-1; k++){
                    if (C->ciclo[i].arestaCinza[k][0] == k+1){//se vertice j forma uma aresta 
                        gerarAresta(j, k, i, i, C, p, revGraph);
                        
                    }
                }
                
            }
        }
    }
    
	
	//comparao entre ciclos
    for(f=0; f<C->nCiclos-1; f++){	
		for(g=f+1; g<C->nCiclos; g++){
            
            for(j=0; j<p->tamanho-1; j++){
                if (C->ciclo[f].arestaCinza[j][0] == j+1){//se vertice j forma uma aresta                
                    
                    for(k=0; k<p->tamanho-1; k++){
                        if (C->ciclo[g].arestaCinza[k][0] == k+1){//se vertice k forma uma aresta 
                            
                            gerarAresta(j, k, f, g, C, p, revGraph);
                            
                        }
                    }
                    
                }
            }            
            
        }
    }
       
	
}//end gerarReversalGraph

void gerarAresta(int vertice_i,int vertice_j,/*int cinza1, int cinza2,int cinza3, int cinza4,*/ 
        int i, int j,decomposicaoCiclos *C,permutacao *p, reversalGraph *revGraph){ 
     
    int a,b,lgu,rgu,lgv,rgv,lbu,rbu,lbv,rbv,adjPreta;
	//comparar se lg(u)<lg(v)<rg(u)<rg(v) o lg(v)<lg(u)<rg(v)<rg(u)
    if (p->piInverso[vertice_i] < p->piInverso[vertice_i+1]){
        lgu = p->piInverso[vertice_i]; 
        rgu = p->piInverso[vertice_i+1]; 
    }
    else{
        lgu = p->piInverso[vertice_i+1]; 
        rgu = p->piInverso[vertice_i]; 
    }
    
    if (p->piInverso[vertice_j] < p->piInverso[vertice_j+1]){
        lgv = p->piInverso[vertice_j]; 
        rgv = p->piInverso[vertice_j+1]; 
    }
    else{
        lgv = p->piInverso[vertice_j+1]; 
        rgv = p->piInverso[vertice_j]; 
    }
						
		
	
	if (((lgu<lgv)&&(lgv<rgu)&&(rgu<rgv))||
		((lgv<lgu)&&(lgu<rgv)&&(rgv<rgu)))
	{
		//criar aresta
		a = rgu;
		if (p->pi[lgu] < p->pi[rgu]) a = lgu;							
		b = rgv;
		if (p->pi[lgv] < p->pi[rgv]) b = lgv;
		
		//revGraph->adj[a][b] = 1;
		//revGraph->adj[b][a] = 1;							
		revGraph->adj[p->pi[a]][p->pi[b]] = 1;
		revGraph->adj[p->pi[b]][p->pi[a]] = 1;
		
		/* JP: nao estava sendo gerado (eliminacao depende desse vetor) */
		revGraph->nadj[p->pi[a]]++;
		revGraph->nadj[p->pi[b]]++;

		//printf("lgu=%d,rgu=%d,lgv=%d,rgv=%d",lgu,rgu,lgv,rgv);
	}				
	else{ //comparar se lb(u)<lb(v)<rb(u)<rb(v) o lb(v)<lb(u)<rb(v)<rb(u)
		adjPreta = escolherArestaPreta(lgu, rgu, i, C,p);//C->ciclo[i].adjPreta[lgu][0];					
		if (lgu < adjPreta) lbu = lgu;
		else lbu = adjPreta;
		
		adjPreta = escolherArestaPreta(rgu, lgu, i, C,p);//C->ciclo[i].adjPreta[rgu][0];					
		if (rgu < adjPreta) rbu = rgu;
		else rbu = adjPreta;
		
		adjPreta = escolherArestaPreta(lgv, rgv, j, C,p);//C->ciclo[j].adjPreta[lgv][0];
		if (lgv < adjPreta) lbv = lgv;
		else lbv = adjPreta;
		
		adjPreta = escolherArestaPreta(rgv, lgv, j, C,p);//C->ciclo[j].adjPreta[rgv][0]; 
		if (rgv < adjPreta) rbv = rgv;
		else rbv = adjPreta;
		
		if (((lbu<lbv)&&(lbv<rbu)&&(rbu<rbv))||
			((lbv<lbu)&&(lbu<rbv)&&(rbv<rbu)))
		{
			//criar aresta								
			a = rgu;
			if (p->pi[lgu] < p->pi[rgu]) a = lgu;							
			b = rgv;
			if (p->pi[lgv] < p->pi[rgv]) b = lgv;
			
			//revGraph->adj[a][b] = 1;
			//revGraph->adj[b][a] = 1;							
			revGraph->adj[p->pi[a]][p->pi[b]] = 1;
			revGraph->adj[p->pi[b]][p->pi[a]] = 1;
			
		/* JP: nao estava sendo gerado (eliminacao depende desse vetor) */
			revGraph->nadj[p->pi[a]]++;
			revGraph->nadj[p->pi[b]]++;
		}															
	}	
}

void calcularCorVertice(int vertice, int i, decomposicaoCiclos *C, permutacao *p, reversalGraph *revGraph){
	int cinza1,cinza2,seta1,seta2,pos,adjPreta1, adjPreta2;
    cinza1 = p->piInverso[vertice];
    cinza2 = p->piInverso[vertice+1];
	//verificar se azul o vermelho (cabea = 1, calda = 0)
	seta1 = 0;//calda
	seta2 = 0;//calda
	
	adjPreta1 = escolherArestaPreta(cinza1,cinza2,i, C,p);//para cinza1
	adjPreta2 = escolherArestaPreta(cinza2,cinza1,i, C,p);//para cinza2
	
	if (cinza1 > adjPreta1)
		seta1 = 1;//cabea
	if (cinza2 > adjPreta2)
		seta2 = 1;//cabea
	
	if (seta1 != seta2){//cabea com calda
		pos = cinza2;
		if (p->pi[cinza1] < p->pi[cinza2]) pos = cinza1;
		
		revGraph->cor[p->pi[pos]] = AZUL;//pintar de cor azul
	}
	else{//cabea com cabea o calda com calda
		pos = cinza2;
		if (p->pi[cinza1] < p->pi[cinza2]) pos = cinza1;
		
		revGraph->cor[p->pi[pos]] = VERMELHO;//pintar de cor vermelho
	}

	revGraph->tamanho++; /* JP: nao estava contando os vertices. Vai contar dobrado aqui */	
}


int escolherArestaPreta(int cinza1, int cinza2, int i,decomposicaoCiclos *C,permutacao *p){
	
    int preta;
    preta = -1;
    //escolher aresta preta de acordo com a orientacao da aresta cinza
    if (p->pi[cinza1] < p->pi[cinza2]){
        preta = p->piInverso[C->ciclo[i].arestaCinza[p->pi[cinza1]][1]];//aresta preta relacionada com cinza1
    }
    else{//p->pi[cinza1] > p->pi[cinza2]
        preta = p->piInverso[C->ciclo[i].arestaCinza[p->pi[cinza2]][2]];//aresta preta relacionada com cinza
    }  
    
    return preta;    
}


