/*
 ============================================================================
 Project     : Genetic algorithm for sorting by reversals with simple fitness function
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
                Universidade de Brasilia (UnB) - Brazil
 
 Last Modified On: January 23, 2012.
 ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h> //malloc, free
#include <time.h>
#include <sys/time.h>//para medir tiempo
#include <math.h> //log2

#include "estructuras.h"
#include "operadores.h"

int flag_pos = 1;//(1 mostra posies e 0 mostra valores, das reverses)//por el momento solo pa reversoes iniciais
int flag_rev_ini = 0;//mostra reverses iniciais
int flag_proceso = 0;//mostrar proceso
int flag_resultado = 1;//mostrar resultado

float prob_crossover = 0.8;//probabilidad de crossover (tiene que ir de menos a mas)
float prob_mutacion = 0.2;//probabilidad de mutacion 

//----------------------------------------------------------------------
void leer_permutacion(permutacion *p){
    int n,i;
    
    //leer permutacion	
    printf("** Genetic Algorithm with simple fitness function **\n");
    printf("Enter permutation length:");
	scanf("%d",&n);  
    
	n = n + 2;
	p->tamanho = n;	
	p->pi[0] = 0;
    p->pi[n-1] = n-1;
    
    for(i=1; i<p->tamanho-1; i++){ 
		printf("pi[%d]? ",i);      
        scanf("%d",&p->pi[i]);  
	}
}

void mostrar_permutacion(const int *p, int tamanho){
int i;
	for (i=0; i<tamanho; i++)
		printf("%d ", p[i]);
	printf("\n");
}

void mostrar_poblacion(poblacion *pob){//*OK
	int i,j;
	for(i=0;i<pob->tamanho;i++){
		for(j=0;j<pob->cromosomas[i].tamanho; j++){
            printf("(%d,%d),", pob->cromosomas[i].genes[j].izq, pob->cromosomas[i].genes[j].der);           
		}
		printf("* fitness: %d , nroBreakpoints: %d\n", pob->cromosomas[i].fitness, pob->cromosomas[i].nroBreakpoints);
	}
}

void mostrarResultado(cromosoma *mejorCromosoma,double secs){//*OK
	int i;
	printf("Final permutation:");
	for (i=0 ; i< mejorCromosoma->p.tamanho; i++)
		printf("%d ", mejorCromosoma->p.pi[i]);
	printf("\n");	 
	//printf("\tNumber of breakpoints:%d\n", mejorCromosoma->nroBreakpoints);
	printf("Number of reversals: %d\n",mejorCromosoma->tamanho);
	
    if (flag_pos)
        printf("Reversals(positions):");
    else
        printf("Reversals(values):");
    for(i=0; i<mejorCromosoma->tamanho-1; i++)
        printf("(%d,%d),", mejorCromosoma->genes[i].izq, mejorCromosoma->genes[i].der);
    printf("(%d,%d)\n",mejorCromosoma->genes[i].izq, mejorCromosoma->genes[i].der);
           
	printf("Total time: %.16g milliseconds\n", secs * 1000.0);
	 printf("\n");
}
//----------------------------------------------------------------------
double timeval_diff(struct timeval *a, struct timeval *b)
{
    return
    (double)(a->tv_sec + (double)a->tv_usec/1000000) -
    (double)(b->tv_sec + (double)b->tv_usec/1000000);
}

int main(){	
    struct timeval t_ini, t_fin;
    double secs;
	
    gettimeofday(&t_ini, NULL);//Tomar TIEMPO de inicio
    
    srand (time(NULL));//initialize random generator
    
	int i,k,n;
	permutacion p;//perm es p.pi
	poblacion pob;
    
	//leer permutacion
    leer_permutacion(&p);
	n = p.tamanho;
	
	//mostrar permutacion inicial
	printf("\nInitial permutation:"); 
	mostrar_permutacion(p.pi, p.tamanho);
    
	int nro_gen = n;//nro de generaciones 		
	pob.tamanho = (int) (n * (log(n) / log(2)));//(nlogn) log base 2
        pob.cromosomas = malloc(pob.tamanho*sizeof(cromosoma));//alocar memoria
	pob.limiteCrossover = 0.15*pob.tamanho;// (0.2) de la poblacion (0.15)
	pob.limiteInferior = 0.4*pob.tamanho;// (0.4) de la poblacion (0.4)	
    
    pob.limiteSuperior = pob.tamanho;	
	
	//generar poblacion inicial
	poblacion_inicial(&p, &pob, flag_rev_ini, flag_pos);
			
	//evaluar fitness de poblacion	
	evaluar_fitness(&p,&pob);
	
	if (flag_proceso){
		printf("\nMATRIZ inicial:\n");
		mostrar_poblacion(&pob);
	}
	
    k = 0;
	float prob;
	cromosoma mejorCromosoma;
    mejorCromosoma.tamanho = n;//se podria inicializar con un nmero ms grande
    mejorCromosoma.nroBreakpoints = n;//se podria inicializar con un nmero ms grande
	
	//EVOLUCION de la poblacion	
	for (i=2; i<=nro_gen; i++){
		//SELECCION: (ordenar) y guardar la mejor solucion
		seleccion(&pob);
			//**Guardar la mejor solucion
            if ((pob.cromosomas[0].tamanho < mejorCromosoma.tamanho)&&
                             (pob.cromosomas[0].nroBreakpoints == 0)){
                mejorCromosoma = pob.cromosomas[0];
                k++;
                
                prob_crossover = 0.8;//(o melhor)ok 1 
                pob.limiteCrossover = 0.3*pob.tamanho;
                pob.limiteInferior = 0.5*pob.tamanho;
            }
                
		if (flag_proceso){
			printf("*****************************************************");
			printf("\n---->GENERACION Nro: %d",i);
			printf("\nmatriz despues de seleccion(sort) (%d):\n",i);	
			mostrar_poblacion(&pob);			
		}
		
		//CROSSOVER
		prob = ((rand() % 10) + 1)/10.0; //de 0.1, 0.2 ... a 1.0
		if (prob <= prob_crossover)
			crossover(&pob);
		 
		if (flag_proceso){
			printf("\nmatriz despues de crossover (%d):\n",i);
			mostrar_poblacion(&pob);
		}
        
        if (mejorCromosoma.nroBreakpoints == 0){//mejorCromosoma.nroBreakpoints == 0
            //MUTACION
            int delta = nro_gen - i + 2;// +2 para conseguir un mejor resultado
            mutacion(&p, &pob, mejorCromosoma.tamanho, delta,flag_pos);
            
            if (flag_proceso){
            	printf("\nmatriz despues de mutacion (%d):    nroBP(%d)\n",i,mejorCromosoma.nroBreakpoints);
            	mostrar_poblacion(&pob);
            }	
        }
			
		//Introduccir NUEVOS GENES en la poblacion 
		nuevos_genes(&p,&pob,flag_pos);
		
		if (flag_proceso){
			printf("\nmatriz despues de nuevos genes (%d):\n",i);
			mostrar_poblacion(&pob);
		}		
		
		//EVALUAR FITNESS de poblacion		
		evaluar_fitness(&p,&pob);
		
		if (flag_proceso){
			printf("\nmatriz despues de eval fitness (%d):\n",i);
			mostrar_poblacion(&pob);			
		}
    }//FIN proceso
    
    //Desalocar memoria
    free(pob.cromosomas);
    
    gettimeofday(&t_fin, NULL);//Tomar TIEMPO final    
    secs = timeval_diff(&t_fin, &t_ini);//Calcular tiempo total usado
    
    //Mostrar resultado final
    if (flag_resultado){
		mostrarResultado(&mejorCromosoma,secs);
	}
 
	return 0;	
}
