/*
 ============================================================================
 Project     : Genetic algorithm for sorting by reversals with simple fitness function
 Author      : Jose Luis Soncco Alvarez and Mauricio Ayala Rincon
                Universidade de Brasilia (UnB) - Brazil
 
 Last Modified On: January 23, 2012.
 ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h> //malloc, free
#include <time.h>
#include <sys/time.h>//para medir tiempo
#include "operadores.h"

//----------------------------------------------------------------------*OK
void swap(int *a, int *b){
	int temp;
	temp = *a;
	*a = *b;
	*b = temp;
}

int dif_breakpoints(permutacion *p, int pos_a, int pos_b){//dif de breakpoints despues de aplicar un reversal a,b 
	int antes = 0, despues = 0;
	
	if (pos_a > pos_b)	
		swap(&pos_a, &pos_b);
	
	//numero de breakpoints antes
	if (abs(p->pi[pos_a - 1] - p->pi[pos_a]) > 1)
		antes++;
	if (abs(p->pi[pos_b] - p->pi[pos_b + 1]) > 1)
		antes++;	
	//numero de breakpoints despues
	if (abs(p->pi[pos_a - 1] - p->pi[pos_b]) > 1)
		despues++;
	if (abs(p->pi[pos_a] - p->pi[pos_b + 1]) > 1)
		despues++;	
	//diferencia de breakpoint
	return despues - antes;	
}
//----------------------------------------------------------------------*OK
/*int determinar_reversals(permutacion *p, gen reversals[], int breakpoints[]){//O(n) 
	int i,j,k,bp;
	//invertir valores e indices
	for(i=0; i<p->tamanho; i++)
		p->piInverso[p->pi[i]] = i;
	
	//determinar reversals
	k=0;
	for(i=1; i < p->tamanho-2; i++){
		for(j=i+1; j < p->tamanho-1; j++){
						
			bp = dif_breakpoints(p, i, j);
			if (bp < 1){//estamos incluyendo los 0-reversals
				if (bp == -2){//hacer 3 copias de 2-reversals
					reversals[k].izq = i; 
					reversals[k].der = j;
					breakpoints[k] = bp;
					k++;
					reversals[k].izq = i; 
					reversals[k].der = j;
					breakpoints[k] = bp;
					k++;
					reversals[k].izq = i; 
					reversals[k].der = j;
					breakpoints[k] = bp;
					k++;					
				}
				else{ // solo una copia cuando son 1-rev o 0-rev
					reversals[k].izq = i; 
					reversals[k].der = j;
					breakpoints[k] = bp;
					k++;
				}							
			}
				
			//ERROR
			if (k == MAX_GENES-1){break; printf("ERROR: Aumentar MAX_GENES");}									
		}			
	}
	return k;//retorna el nro de reversals generado	
}*/

int determinar_reversals(permutacion *p, gen reversals[], int breakpoints[]){
	int i,j,x,y,bp;
	//invertir valores e indices
	for(i=0; i<p->tamanho; i++)
		p->piInverso[p->pi[i]] = i;
	
	//determinar reversals
	j=0;
	for(i=0; i<p->tamanho-1; i++){
		if (abs(p->piInverso[i+1] - p->piInverso[i]) > 1){ //si la dif de pos es mayor a 1
			if (i==0){//si comienza con cero
				reversals[j].izq = 1; 
				reversals[j].der = p->piInverso[i+1];
				breakpoints[j] = dif_breakpoints(p, 1, p->piInverso[i+1]);
				j++;
			}
			else if (i+1==p->tamanho-1){//si termina con el ultimo elemento
				reversals[j].izq = p->piInverso[i]; 
				reversals[j].der = p->tamanho-2;
				breakpoints[j] = dif_breakpoints(p, p->piInverso[i], p->tamanho-2);
				j++;
			}
			else{//Generar el reversal y los otros 2 reversals derivados
				x = i;
				y = i + 1;
				if (p->piInverso[x] > p->piInverso[y]) //si posiciones estan en orden descendente
					swap(&x, &y);				
				
				bp = dif_breakpoints(p, p->piInverso[x], p->piInverso[y]);
				if (bp < 1){
					reversals[j].izq = p->piInverso[x]; 
					reversals[j].der = p->piInverso[y];
					breakpoints[j] = bp;
					j++;
				}
				
				bp = dif_breakpoints(p, p->piInverso[x], p->piInverso[y]-1);
				if (bp < 1){
					reversals[j].izq = p->piInverso[x]; 
					reversals[j].der = p->piInverso[y]-1;
					breakpoints[j] = bp;
					j++;
				}				
				
				bp = dif_breakpoints(p, p->piInverso[x]+1, p->piInverso[y]);					
				if (bp < 1){
					reversals[j].izq = p->piInverso[x]+1; 
					reversals[j].der = p->piInverso[y];				
					breakpoints[j] = bp;					
					j++;
				}
				
				//ERROR
				if (j == MAX_GENES-1){break; printf("ERROR: Aumentar MAX_GENES");}				
			}			
		}			
	}
	return j;//retorna el nro de reversals generado	
} 

//----------------------------------------------------------------------*OK
void aplicar_reversal(poblacion *pob, int pos, int pos_a, int pos_b){	
	int i;
			
	if (pos_a > pos_b)
		swap(&pos_a, &pos_b);
				
	//revertir
	int aux;
	for(i=0; i<(pos_b - pos_a + 1)/2; i++){
		aux = pob->cromosomas[pos].p.pi[pos_a + i];
		pob->cromosomas[pos].p.pi[pos_a + i] = pob->cromosomas[pos].p.pi[pos_b - i];
		pob->cromosomas[pos].p.pi[pos_b - i] = aux;
	}
		
}
//----------------------------------------------------------------------*OK
void poblacion_inicial(permutacion *p, poblacion *pob, int flag_rev_ini, int flag_pos){
	
	int i,rd;
	//inicializar matriz de permutaciones
	for(i=0; i < pob->tamanho; i++){//para cada cromosoma        
        pob->cromosomas[i].p = *p;
	}	
	
	gen reversals[MAX_GENES];//vector con los todos los reversals de p
	int breakpoints[MAX_GENES];//vector con la dif del nro de breakpoints al aplicar un reversal(-1 es buen valor)
	//Determinar todos reversals de la permutacion actual
	int nro_reversals = determinar_reversals(p,reversals,breakpoints);
	
	//mostrar reversals
	if (flag_rev_ini){
		int h;
        if (flag_pos){
            printf("Reverses da permutao inicial (pos)\n");
            for(h=0; h<nro_reversals; h++){
                printf("ro(%d,%d):[%d],\n ",reversals[h].izq, reversals[h].der, breakpoints[h]);
            }
        }
        else{        
            printf("Reverses da permutao inicial (valores)\n");
            for(h=0; h<nro_reversals; h++){
                printf("ro(%d,%d):[%d],\n ",p->pi[reversals[h].izq], p->pi[reversals[h].der], breakpoints[h]);
            }
        }
	}		
	//Generar poblacion inicial a partir de reversals
	if (nro_reversals > 0){//Si hay por lo menos un reversal
		for(i=0; i<pob->tamanho; i++){			
			
			rd = rand() % nro_reversals; //de 0 a nro_reversals-1
				//aqui en un bucle se podria buscar q elimine un breakp por lo menos.
            
            if (flag_pos){//guarda (posiciones) de las reversiones            
                pob->cromosomas[i].genes[0].izq = reversals[rd].izq;
                pob->cromosomas[i].genes[0].der = reversals[rd].der;
                pob->cromosomas[i].tamanho = 1;
            }
            else{//guarda (valores) de las reversiones
                pob->cromosomas[i].genes[0].izq = p->pi[reversals[rd].izq];
                pob->cromosomas[i].genes[0].der = p->pi[reversals[rd].der];
                pob->cromosomas[i].tamanho = 1;
            }
			
			//aplicar reversal 
			aplicar_reversal(pob, i, reversals[rd].izq, reversals[rd].der);			
		}
	}	
	//return i;	
	pob->tamanho = i;
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------*OK
void evaluar_fitness(permutacion *p, poblacion *pob){
	//*el fitness sera el numero de breakpoints, y debera ser minimizado (podria ser + longitud de solucion)			
	int i,j;	
	for(i=0; i<pob->tamanho; i++){
		pob->cromosomas[i].nroBreakpoints = 0;		
		//contar el numero de breakpoints
		for(j=0; j<p->tamanho-1; j++){
			if (abs(pob->cromosomas[i].p.pi[j] - pob->cromosomas[i].p.pi[j+1]) > 1)
				pob->cromosomas[i].nroBreakpoints++;				
		}
		//fitness = nroBreakpoints + tamanho de cromosoma
		pob->cromosomas[i].fitness = pob->cromosomas[i].nroBreakpoints + pob->cromosomas[i].tamanho;				
	}	
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------*OK
void swap2(poblacion *pob,int i, int j) {    
    cromosoma cromosomaT;
    cromosomaT = pob->cromosomas[i];
    pob->cromosomas[i] = pob->cromosomas[j];
    pob->cromosomas[j] = cromosomaT;    
}

int partition(poblacion *pob, int left, int right) {
    int i, j;
    
    i = left;
    for (j = left + 1; j <= right; ++j) {
        if (pob->cromosomas[j].fitness < pob->cromosomas[left].fitness) {
            ++i;
            swap2(pob,i,j);
        }
    }
    swap2(pob,left,i);
    
    return i;
}

void quickSort(poblacion *pob, int left, int right) {
    int r;
    
    if (right > left) {
        r = partition(pob, left, right);
        quickSort(pob, left, r - 1);
        quickSort(pob, r + 1, right);
    }
}
//----------------------------------------------------------------------
void seleccion(poblacion *pob){//O(nlogn)?
	//ordena la poblacion de acuerdo al fitness
    quickSort(pob,0,pob->tamanho-1);
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------*OK
void crossover(poblacion *pob){
	int i,rd;
		
	//(eliminar y) Sobreescribir desde limiteInferior a limiteSuperior
	for(i=pob->limiteInferior; i<pob->limiteSuperior; i++){
		rd = rand() % pob->limiteCrossover; //de 0 a pob.limiteCrossover-1						
		pob->cromosomas[i] = pob->cromosomas[rd];
	}
	
	pob->tamanho = pob->limiteSuperior;	
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------
void mutacion(permutacion *p, poblacion *pob, int tamanhoLimite, int delta,int flag_pos){
	int i,j,k,rd;
				
	//hacer la mutacion en los nuevos elementos producidos por el crossover
	for(i=0; i<pob->limiteSuperior; i++){
        //mutacion si cromosoma tiene el tamanho limite de mejor solucion encontrada
        if (pob->cromosomas[i].tamanho == tamanhoLimite){
            rd = (rand() % delta) + 1;//de 1 a delta		
            //printf("rd:%d, prob:%f\n",rd,prob);
            
            //asignar nuevo tamanho de soluciones
            pob->cromosomas[i].tamanho = pob->cromosomas[i].tamanho - rd;
            if (pob->cromosomas[i].tamanho <= 0) pob->cromosomas[i].tamanho = 1;
            
            //copiar permutacion original sobre solucion actual            
            pob->cromosomas[i].p = *p;
            
            //reaplicar reversiones de solucion actual sobre permutacion original
            for (j=0; j < pob->cromosomas[i].tamanho; j++){
                if (flag_pos){//cromosoma de posiciones
                    //aplicar reversal
                    aplicar_reversal(pob, i, pob->cromosomas[i].genes[j].izq, pob->cromosomas[i].genes[j].der);
                }
                else{//cromosoma de valores
                    //invertir valores e indices
                    for(k=0; k < p->tamanho; k++)
                        pob->cromosomas[i].p.piInverso[pob->cromosomas[i].p.pi[k]] = k;
                    //aplicar reversal
                    aplicar_reversal(pob, i, pob->cromosomas[i].p.piInverso[pob->cromosomas[i].genes[j].izq],
                                               pob->cromosomas[i].p.piInverso[pob->cromosomas[i].genes[j].der]);
                
                }
                
            }
        }
	}
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------*OK
void nuevos_genes(permutacion *p, poblacion *pob, int flag_pos){
	int i,rd;
		
	gen reversals[MAX_GENES];//vector con los todos los reversals de p
	int breakpoints[MAX_GENES];//vector con la dif del nro de breakpoints al aplicar un reversal(-1 es buen valor)
			
	for(i=0; i<pob->tamanho; i++){		
		//determinar todos los reversals de permutacion de cromosoma i
		int nro_reversals = determinar_reversals(&pob->cromosomas[i].p,reversals,breakpoints);	
		//Generar poblacion inicial a partir de reversals
		if (nro_reversals > 0){			
			//elegir un reversal (o el mejor)
			rd = rand() % nro_reversals; //de 0 a nro_reversals-1	
            
            if (flag_pos){//guarda (posiciones) de las reversiones            
                pob->cromosomas[i].genes[pob->cromosomas[i].tamanho].izq = reversals[rd].izq;
                pob->cromosomas[i].genes[pob->cromosomas[i].tamanho].der = reversals[rd].der;
                pob->cromosomas[i].tamanho++;
            }
            else{//guarda (valores) de las reversiones
                pob->cromosomas[i].genes[pob->cromosomas[i].tamanho].izq = pob->cromosomas[i].p.pi[reversals[rd].izq];
                pob->cromosomas[i].genes[pob->cromosomas[i].tamanho].der = pob->cromosomas[i].p.pi[reversals[rd].der];
                pob->cromosomas[i].tamanho++;
            }
            
			//aplicar reversal en permutacion
			aplicar_reversal(pob, i, reversals[rd].izq, reversals[rd].der);								
		}		
	}	
}
